import datetime, os
import gobject, gtk, gtk.glade
import Pygem.conf as conf

import Pygem.backend as be, Pygem.backend.doc as doc, Pygem.backend.sendmail as mail
import main_widgets

        
class MainWindow:
    def __init__(self):
        self.ui = gtk.glade.XML(conf.ui, "main_window")
        self.ui.signal_autoconnect(self)

        self.member_list = main_widgets.MemberList(self.ui.get_widget("member_list"))
        
        self.set_members_filter(conf.members_at_startup)
        
        print "pyGEM ready"

    def show_credits(self, *args):
        dia = CreditsDialog()
        dia.run()
        dia.destroy()
        
    def on_quit(self, *args):
        gtk.main_quit()

    def on_add_member_clicked(self, *args):
        dialog = AddMemberDialog()
        dialog.run()
        self.member_list.populate()
        dialog.destroy()

    def on_del_member_clicked(self, *args):
        sel = self.member_list.get_selected()
        self.member_list.delete(sel)

    def on_edit_member_clicked(self, *args):
        sel = self.member_list.get_selected()
        model, iter = sel.get_selected()
        if iter:
            id = model.get_value(iter, 0)
            dialog = EditMemberDialog(id)
            dialog.run()
            self.member_list.populate()
            dialog.destroy()

    def on_switch_notyet_members(self, *args):
        self.set_members_filter("NotYet")

    def on_switch_active_members(self, *args):
        self.set_members_filter("Active")

    def on_switch_old_members(self, *args):
        self.set_members_filter("Old")

    def on_switch_all_members(self, *args):
        self.set_members_filter(None)

    def set_members_filter(self, filter):
        self.member_list.set_filter(filter)
        self.member_list.populate()
        
    def on_refresh_clicked(self, *args):
        self.member_list.populate()


class CreditsDialog:
    def __init__(self):
        self.ui = gtk.glade.XML(conf.ui, "about_dialog")
        self.ui.signal_autoconnect(self)
        self.dialog = self.ui.get_widget("about_dialog")

    def run(self):
        self.dialog.run()

    def destroy(self):
        self.dialog.destroy()
    
class EditMemberDialog:
    def __init__(self, memberid):
        self.ui = gtk.glade.XML(conf.ui, "edit_member_dialog")
        self.ui.signal_autoconnect(self)
        self.dialog = self.ui.get_widget("edit_member_dialog")

        self.member = be.Member.get(memberid)

        self.fill()

    def on_cancel_clicked(self, *args):
        self.destroy()

    def on_ok_clicked(self, *args):
        self.save()
        self.destroy()

    def save(self):
        # Identity
        name = self.ui.get_widget("name_entry")
        firstname = self.ui.get_widget("firstname_entry")
        nickname = self.ui.get_widget("nickname_entry")
        birthdate_day = self.ui.get_widget("birthdate_day")
        birthdate_month = self.ui.get_widget("birthdate_month")
        birthdate_year = self.ui.get_widget("birthdate_year")
        address = self.ui.get_widget("address_entry")
        postalcode = self.ui.get_widget("postalcode_entry")
        town = self.ui.get_widget("town_entry")
        addrcom = self.ui.get_widget("addrcom_entry")
        country = self.ui.get_widget("country_entry")
        email = self.ui.get_widget("email_entry")
        phone = self.ui.get_widget("phone_entry")

        birthdate = datetime.date(int(birthdate_year.get_text()),
                                  int(birthdate_month.get_text()),
                                  int(birthdate_day.get_text())
                                  )

        self.member.Name = name.get_text()
        self.member.Firstname = firstname.get_text()
        self.member.Nickname = nickname.get_text()
        self.member.Birthdate = birthdate
        self.member.Address = address.get_text()
        self.member.PostalCode = postalcode.get_text()
        self.member.Town = town.get_text()
        self.member.AddressComp = addrcom.get_text()
        self.member.Country = country.get_text()
        self.member.Email = email.get_text()
        self.member.Phone = phone.get_text()
        
        # Association
        member_type = self.ui.get_widget("member_notyet")
        reg_day = self.ui.get_widget("reg_day")
        reg_month = self.ui.get_widget("reg_month")
        reg_year = self.ui.get_widget("reg_year")
        renew_day = self.ui.get_widget("renew_day")
        renew_month = self.ui.get_widget("renew_month")
        renew_year = self.ui.get_widget("renew_year")
        contrib = self.ui.get_widget("contrib")
        has_mailbox = self.ui.get_widget("has_mailbox")
        has_web = self.ui.get_widget("has_web")
        paymode = self.ui.get_widget("paymode_check")
        
        regdate = datetime.date(int(reg_year.get_text()),
                                int(reg_month.get_text()),
                                int(reg_day.get_text())
                                )
        renewdate = datetime.date(int(renew_year.get_text()),
                                  int(renew_month.get_text()),
                                  int(renew_day.get_text())
                                  )

        self.member.RegDate = regdate
        self.member.RenewDate = renewdate
        self.member.Contribution = contrib.get_value()

        for mtype in member_type.get_group():
            if mtype.get_active():
                if mtype.name == "member_notyet":
                    self.member.Type = "NotYet"
                elif mtype.name == "member_active":
                    self.member.Type = "Active"
                elif mtype.name == "member_old":
                    self.member.Type = "Old"

        for mode in paymode.get_group():
            if mode.get_active():
                if mode.name == "paymode_check":
                    self.member.PayMode = "Check"
                elif mode.name == "paymode_cash":
                    self.member.PayMode = "Cash"
                else:
                    self.member.PayMode = "Other"

        self.member.Mailbox = has_mailbox.get_active()
        self.member.WebPage = has_web.get_active()

        # Notes
        notes = self.ui.get_widget("notes")
        buff = notes.get_buffer()
        start = buff.get_start_iter()
        end = buff.get_end_iter()
        text = buff.get_text(start, end)
        self.member.Notes = text


    def fill(self):
        # Identity
        name = self.ui.get_widget("name_entry")
        firstname = self.ui.get_widget("firstname_entry")
        nickname = self.ui.get_widget("nickname_entry")
        birthdate_day = self.ui.get_widget("birthdate_day")
        birthdate_month = self.ui.get_widget("birthdate_month")
        birthdate_year = self.ui.get_widget("birthdate_year")
        address = self.ui.get_widget("address_entry")
        postalcode = self.ui.get_widget("postalcode_entry")
        town = self.ui.get_widget("town_entry")
        addrcom = self.ui.get_widget("addrcom_entry")
        country = self.ui.get_widget("country_entry")
        email = self.ui.get_widget("email_entry")
        phone = self.ui.get_widget("phone_entry")

        name.set_text(self.member.Name)
        firstname.set_text(self.member.Firstname)
        nickname.set_text(self.member.Nickname)
        birthdate_day.set_value(self.member.Birthdate.day)
        birthdate_month.set_value(self.member.Birthdate.month)
        birthdate_year.set_value(self.member.Birthdate.year)
        address.set_text(self.member.Address)
        postalcode.set_text(self.member.PostalCode)
        town.set_text(self.member.Town)
        addrcom.set_text(self.member.AddressComp)
        country.set_text(self.member.Country)
        email.set_text(self.member.Email)
        phone.set_text(self.member.Phone)

        # Association
        member_type = self.ui.get_widget("member_notyet")
        reg_day = self.ui.get_widget("reg_day")
        reg_month = self.ui.get_widget("reg_month")
        reg_year = self.ui.get_widget("reg_year")
        renew_day = self.ui.get_widget("renew_day")
        renew_month = self.ui.get_widget("renew_month")
        renew_year = self.ui.get_widget("renew_year")
        contrib = self.ui.get_widget("contrib")
        has_mailbox = self.ui.get_widget("has_mailbox")
        has_web = self.ui.get_widget("has_web")

        paymode = self.ui.get_widget("paymode_check")

        for type in member_type.get_group():
            if type.name == "member_notyet":
                if self.member.Type is "NotYet":
                    type.set_active(True)

            elif type.name == "member_active":
                if self.member.Type is "Active":
                    type.set_active(True)

            elif type.name == "member_old":
                if self.member.Type is "Old":
                    type.set_active(True)

        reg_day.set_value(int(self.member.RegDate.day))
        reg_month.set_value(int(self.member.RegDate.month))
        reg_year.set_value(int(self.member.RegDate.year))

        renew_day.set_value(int(self.member.RenewDate.day))
        renew_month.set_value(int(self.member.RenewDate.month))
        renew_year.set_value(int(self.member.RenewDate.year))

        contrib.set_value(float(self.member.Contribution))

        for mode in paymode.get_group():
            if mode.get_label() == "paymode_check":
                if self.member.PayMode == "Check":
                    mode.set_active(True)
            
            if mode.name == "paymode_cash":
                if self.member.PayMode == "Cash":
                    mode.set_active(True)

            if mode.name == "paymode_other":
                if self.member.PayMode == "Other":
                    mode.set_active(True)

        if self.member.Mailbox:
            has_mailbox.set_active(True)

        if self.member.WebPage:
            has_web.set_active(True)

        # Notes
        notes = self.ui.get_widget("notes")
        buffer = self.member.Notes
        textbuffer = gtk.TextBuffer()
        iter = textbuffer.get_iter_at_offset(0)
        textbuffer.insert_with_tags_by_name(iter, buffer)
        notes.set_buffer(textbuffer)


    def run(self):
        self.dialog.run()

    def destroy(self):
        self.dialog.destroy()

    def on_reg_today_clicked(self, *args):
        reg_day = self.ui.get_widget("reg_day")
        reg_month = self.ui.get_widget("reg_month")
        reg_year = self.ui.get_widget("reg_year")

        now = datetime.date(1900, 01, 01)
        now = now.today()

        reg_day.set_value(now.day)
        reg_month.set_value(now.month)
        reg_year.set_value(now.year)
        
    
    def on_renew_today_clicked(self, *args):
        renew_day = self.ui.get_widget("renew_day")
        renew_month = self.ui.get_widget("renew_month")
        renew_year = self.ui.get_widget("renew_year")

        now = datetime.date(1900, 01, 01)
        now = now.today()

        renew_day.set_value(now.day)
        renew_month.set_value(now.month)
        renew_year.set_value(now.year)


    def on_form_adh_gen_clicked(self, *args):
        d = be.doc.Latex2Pdf("res/adhesion.tex")

        rep = dict()
        rep["Firstname"] = self.member.Firstname
        rep["Name"] = self.member.Name
        rep["Address"] = self.member.Address
        rep["PostalCode"] = self.member.PostalCode
        rep["Town"] = self.member.Town
        rep["Email"] = self.member.Email
        rep["AddressComp"] = self.member.AddressComp
        rep["Country"] = self.member.Country
        filename = d.generate(rep)

        os.spawnlp(os.P_NOWAIT, conf.pdf_viewer, conf.pdf_viewer, filename)


    def on_renew_email_clicked(self, *args):
        fd = open("res/renew_email", "r")
        raw = fd.readlines()
        fd.close()

        rep = dict()
        rep["Firstname"] = self.member.Firstname
        rep["RenewDate"] = str(self.member.RenewDate)
        rep["Contribution"] = str(self.member.Contribution)


        # Replace each dico key by its value
        for k in range(0, len(raw)):
            for elt in rep.keys():
                raw[k] = raw[k].replace("$PYGEM:%s$" %elt, rep[elt].encode("iso-8859-1"))

        body = ""
        for l in raw:
            body += l
        
        email = mail.SendMail(self.member.Email, "Renouvellement d'adhsion  Linux62", body)
        email.loadMail()

    def on_renew_mail_clicked(self, *args):
        d = be.doc.Latex2Pdf("res/renew.tex")

        rep = dict()
        rep["Firstname"] = self.member.Firstname
        rep["Name"] = self.member.Name
        rep["Address"] = self.member.Address
        rep["PostalCode"] = self.member.PostalCode
        rep["Town"] = self.member.Town
        rep["Email"] = self.member.Email
        rep["AddressComp"] = self.member.AddressComp
        rep["Country"] = self.member.Country
        rep["RenewDate"] = str(self.member.RenewDate)
        filename = d.generate(rep)

        os.spawnlp(os.P_NOWAIT, conf.pdf_viewer, conf.pdf_viewer, filename)


        
        

        
        

class AddMemberDialog:
    def __init__(self):
        self.ui = gtk.glade.XML(conf.ui, "add_member_dialog")
        self.ui.signal_autoconnect(self)
        self.dialog = self.ui.get_widget("add_member_dialog")

    def run(self):
        self.dialog.run()

    def destroy(self):
        self.dialog.destroy()
        
    def on_cancel_clicked(self, *args):
        self.destroy()


    def on_ok_clicked(self, *args):
        name = self.ui.get_widget("name_entry")
        firstname = self.ui.get_widget("firstname_entry")
        nickname = self.ui.get_widget("nickname_entry")
        birthdate_day = self.ui.get_widget("birthdate_day")
        birthdate_month = self.ui.get_widget("birthdate_month")
        birthdate_year = self.ui.get_widget("birthdate_year")
        address = self.ui.get_widget("address_entry")
        postalcode = self.ui.get_widget("postalcode_entry")
        town = self.ui.get_widget("town_entry")
        addrcom = self.ui.get_widget("addrcom_entry")
        country = self.ui.get_widget("country_entry")
        email = self.ui.get_widget("email_entry")
        phone = self.ui.get_widget("phone_entry")

        birthdate = str(birthdate_year.get_value_as_int()) + "-" + str(birthdate_month.get_value_as_int()) + "-" + str(birthdate_day.get_value_as_int())
        
        be.Member(Name=name.get_text(),
                  Firstname=firstname.get_text(),
                  Nickname=nickname.get_text(),
                  Birthdate = birthdate,
                  Address = address.get_text(),
                  PostalCode = postalcode.get_text(),
                  Town=town.get_text(),
                  AddressComp = addrcom.get_text(),
                  Country = country.get_text(),
                  Email = email.get_text(),
                  Phone = phone.get_text()
                  )

        self.destroy()
        
        

